/*
 * smtp - Firestorm SMTP decode plugin
 * Copyright (c) 2002 Gianni Tedesco <gianni@scaramanga.co.uk>
 * Released under the terms of the GNU GPL v2
 *
 * TODO:
 *  o Track application layer state
 *  o Parse responses and addresses
 *  o Matchers
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <firestorm.h>
#include <args.h>
#include <packet.h>
#include <plugin.h>
#include <alert.h>
#include <signature.h>
#include <decode.h>
#include <preproc.h>
#include <netinet/in.h>

PLUGIN_STD_DEFS();
static proc_dispatch dispatch;

/* Returns number of bytes fit to proces */
static size_t
smtp_sdecode(struct packet *p, char *buf, size_t len)
{
	char *end;
	size_t ret = 0;
	int i;

	if ( buf[len-1] == '\n' )
		return len;
	
	for(end=buf+len, i=0; buf < end; buf++, i++) {
		if ( *buf == '\n' )
			ret = i + 1;
	}

	return ret;
}

void smtp_response_decode(struct packet *p, struct fvec *v)
{
	unsigned int code;

	/* Check we have a 3 digit code */
	if ( v->len < 3 )
		return;

	if ( vtouint(v, &code) != 3 )
		return;

	/* TODO: Attach session info */
}

void smtp_request_decode(struct packet *p, struct fvec *v)
{
	struct fvec cmd;
	struct fvec val;
	int state = 0;
	char *ptr;
	size_t i;
	
	cmd.ptr = ptr = v->ptr;
	cmd.len = 0;
	val.ptr = NULL;
	val.len = 0;

	for(i=0; i < v->len; i++, ptr++) {
		if ( state == 0 ) {
			if ( *ptr == ' ' ) {
				cmd.len = i;
				state = 1;
			}
		} else {
			if ( *ptr != ' ' && *ptr != '\t' ) {
				val.ptr = ptr;
				val.len = v->len - i;
				break;
			}
		}
	}

	if ( cmd.len == 0 )
		return;
}

/* SMTP decode callback */
static void smtp_decode(struct packet *p)
{
	struct fvec v;
	char *begin;
	char *end;

	begin = p->layer[p->llen].h.raw;
	end = p->layer[p->llen-2].h.raw +
		ntohs(p->layer[p->llen-2].h.ip->tot_len);

	/* TODO: Support pipelining by splitting on CRLFs */

	/* Trim trailing CRLF */
	if ( *(end-1) == '\n' )
		end--;
	if ( *(end-1) == '\r' )
		end--;

	v.ptr = begin;
	v.len = end - begin;

	if ( p->layer[p->llen-1].flags & FLAG_TCP_2SVR ) {
		smtp_request_decode(p, &v);
	}else{
		smtp_response_decode(p, &v);
	}

	dispatch(p);
}

struct proto smtp_p={
	.name="smtp",
	.sdecode=smtp_sdecode,
	.decode=smtp_decode,
	.flow_len=0,
};

struct proto_req smtp_r[]={
	proto_request("tcp", __constant_htons(25)),
	null_request()
};

int PLUGIN_DECODE (struct decode_api *d)
{
	int ok=0;

	object_check(d);

	dispatch=d->dispatch;

	ok+=d->decode_add(&smtp_p, smtp_r);

	return (ok) ? PLUGIN_ERR_OK : PLUGIN_ERR_FAIL;
}

int PLUGIN_INIT (struct plugin_in *in, struct plugin_out *out)
{
	plugin_check(in, out);

	PLUGIN_ID("decode.smtp", "Simple Mail Transfer Protocol");
	PLUGIN_VERSION(0, 1);
	PLUGIN_AUTHOR("Gianni Tedesco", "gianni@scaramanga.co.uk");
	PLUGIN_LICENSE("GPL");

	return PLUGIN_ERR_OK;
}

int PLUGIN_UNLOAD (int code) {
	return PLUGIN_ERR_OK;
}
