#include "tcpip.h"

static int get_icmp_type(struct packet *p, struct alert *a, int l, idx_key_t *k)
{
	*k = (idx_key_t)p->layer[l].h.icmp->type;
	return 1;
}

static int get_icmp_code(struct packet *p, struct alert *a, int l, idx_key_t *k)
{
	*k = (idx_key_t)p->layer[l].h.icmp->code;
	return 1;
}

static int get_udp_sport(struct packet *p, struct alert *a, int l, idx_key_t *k)
{
	*k = (idx_key_t)p->layer[l].h.udp->sport;
	return 1;
}

static int get_udp_dport(struct packet *p, struct alert *a, int l, idx_key_t *k)
{
	*k = (idx_key_t)p->layer[l].h.udp->dport;
	return 1;
}

static int get_tcp_sport(struct packet *p, struct alert *a, int l, idx_key_t *k)
{
	*k = (idx_key_t)p->layer[l].h.tcp->sport;
	return 1;
}

static int get_tcp_dport(struct packet *p, struct alert *a, int l, idx_key_t *k)
{
	*k = (idx_key_t)p->layer[l].h.tcp->dport;
	return 1;
}

static int get_ip_src(struct packet *p, struct alert *a, int l, idx_key_t *k)
{
	*k = (idx_key_t)p->layer[l].h.ip->saddr;
	return 1;
}

static int get_ip_dst(struct packet *p, struct alert *a, int l, idx_key_t *k)
{
	*k = (idx_key_t)p->layer[l].h.ip->daddr;
	return 1;
}

static int get_ip_proto(struct packet *p, struct alert *a, int l, idx_key_t *k)
{
	*k = (idx_key_t)p->layer[l].h.ip->protocol;
	return 1;
}

static int get_ip_ttl(struct packet *p, struct alert *a, int l, idx_key_t *k)
{
	*k = (idx_key_t)p->layer[l].h.ip->ttl;
	return 1;
}

struct proto_field ip_fields[]={
	{
		.name = "ip.src",
		.type = FIELD_TYPE_NUM,
		.get_key = {.num = get_ip_src},
	},
	{
		.name = "ip.dst",
		.type = FIELD_TYPE_NUM,
		.get_key = {.num = get_ip_dst},
	},
	{
		.name = "ip.ttl",
		.type = FIELD_TYPE_NUM,
		.get_key = {.num = get_ip_ttl},
	},
	{
		.name = "ip.proto",
		.type = FIELD_TYPE_NUM,
		.get_key = {.num = get_ip_proto},
	},
	{.name = NULL}
};


struct proto_field tcp_fields[]={
	{
		.name = "tcp.dport",
		.type = FIELD_TYPE_NUM,
		.get_key = {.num = get_tcp_dport},
	},
	{
		.name = "tcp.sport",
		.type = FIELD_TYPE_NUM,
		.get_key = {.num = get_tcp_sport},
	},
	{.name = NULL}
};

struct proto_field udp_fields[]={
	{
		.name = "udp.dport",
		.type = FIELD_TYPE_NUM,
		.get_key = {.num = get_udp_dport},
	},
	{
		.name = "udp.sport",
		.type = FIELD_TYPE_NUM,
		.get_key = {.num = get_udp_sport},
	},
	{.name = NULL}
};

struct proto_field icmp_fields[]={
	{
		.name = "icmp.type",
		.type = FIELD_TYPE_NUM,
		.get_key = {.num = get_icmp_type},
	},
	{
		.name = "icmp.code",
		.type = FIELD_TYPE_NUM,
		.get_key = {.num = get_icmp_code},
	},
	{.name = NULL}
};
